// 用于不能加载插件的Linux上面的BDS来监听玩家聊天和实现一些简单的命令功能
// 实现了玩家聊天和QQ群聊天互通
// 注意事项：请将bot授予操作员权限并且设置为观察者模式（如果不设置为观察者模式会影响玩家睡觉）
// By: KangKang (2646381627)
// 版本号v0.0.2
// ChangeLog:
// 0.0.2: 添加了QQ群互通的功能(使用mirai-js)
var os = require('os')
var fs = require('fs')
const date = require('date-and-time')
const bedrock = require('bedrock-protocol')

var config = JSON.parse(fs.readFileSync("config.json").toString())

const client = bedrock.createClient({
  host: config.bedrockServer.ip,
  port: config.bedrockServer.port,
  offline: !config.bedrockServer.onlineMode
})
const uuid = require("uuid");
var homes = fs.readFileSync("homes.json")
if (homes == undefined) {
    homes = {}
}
else{
    homes = JSON.parse(homes)
}
var translationMap = {}
var translationLangs = fs.readFileSync("translation.lang").toString().split(os.EOL)
for (var i = 0; i < translationLangs.length; i++) {
    let singleTranslationLangs = translationLangs[i].split("=")
    translationMap[singleTranslationLangs[0]] = singleTranslationLangs[1].replaceAll(os.EOL, "")
}

const { Bot, Message, Middleware  } = require('mirai-js')
const bot = new Bot();
const middleware = new Middleware();

async function qqBotInit() {
    await bot.open({
        baseUrl: config.mirai.httpApiUrl,
        verifyKey: config.mirai.verifyKey,
        qq: config.mirai.qqNumber,
    });
}
qqBotInit()

function sendText(text){
    client.queue('text', 
            {
                type: 'chat', 
                needs_translation: false, 
                source_name: client.username, 
                xuid: '', 
                platform_chat_id: '', 
                message: text
            })
}

function sendCommand(text){
    client.queue('command_request',
            {
                command: text,
                origin: {
                    type: 0,
                    uuid: uuid.v4(),
                    request_id: uuid.v4()
                },
                interval: false
            })
}

function sendMsg(playerName, text){
    sendCommand(`tellraw ${playerName} {"rawtext":[{"text":"§f${text}"}]}`)
    //console.log(`给 ${playerName} 发送了 ${text}`)
}

function sendTip(playerName, text){
    sendCommand(`tellraw ${playerName} {"rawtext":[{"text":"§2${text}"}]}`)
    //console.log(`给 ${playerName} 提示了 ${text}`)
}

function sendWarn(playerName, text){
    sendCommand(`tellraw ${playerName} {"rawtext":[{"text":"§e${text}"}]}`)
    console.warn(`给 ${playerName} 警告了 ${text}`)
}

function sendErr(playerName, text){
    sendCommand(`tellraw ${playerName} {"rawtext":[{"text":"§4${text}"}]}`)
    console.error(`给 ${playerName} 报错了 ${text}`)
}

function sendQQMsg(sender, text){
    sendCommand(`tellraw @a[name=!client.name] {"rawtext":[{"text":"§e[QQ]§7<${sender}> ${text}"}]}`)
}

function giveItem(targerName, itemName, amount, date = 0){
    sendCommand(`give ${targerName} ${itemName} ${amount} ${date}`)
}

function tpToPosition(playerName, position){
    sendCommand(`tp ${playerName} ${position.x} ${position.y} ${position.z}`)
}

function botSendMessage(message){
    bot.sendMessage({
            group: config.mirai.groupNumber,
            message: [
                { type: 'Plain', text: message },
            ],
        });
}

bot.on('GroupMessage', new Middleware()
    .textProcessor()
    .done(async data => {
        if (data.sender.group.id == config.mirai.groupNumber && data.text != "") {
            if (data.text.startsWith("/")) {
                if (data.text == "/list") {
                    sendCommand("list")
                }
            }
            else{
                sendQQMsg(data.sender.memberName, data.text)
            }
        }
    })
)

client.on('text', (packet) => { // Listen for chat messages and echo them back.
    let now  =  new Date();
    //console.log(packet, JSON.stringify(packet));
    let message = `[${now.toLocaleString()}]${packet.source_name}: ${packet.message}`
    if(packet.needs_translation){
        message = `[${now.toLocaleString()}]${packet.message} 关于 ${packet.parameters}`
        console.log(message)
        //let translatedMessage = translationMap[packet.message]
        if (packet.message.startsWith("§e%multiplayer.player.left")) {
            botSendMessage(`${packet.parameters} 离开了游戏`)
        }
        else if (packet.message.startsWith("§e%multiplayer.player.joined")) {
            botSendMessage(`${packet.parameters} 加入了游戏`)
        }
        else {
            if (translationMap[packet.message] != undefined) {
                let translatedMessage = translationMap[packet.message]
                for (var i = 0; i < packet.parameters.length; i++) {
                    let parameter = packet.parameters[i].replaceAll("%", "")
                    if (translationMap[parameter] != undefined) {
                        parameter = translationMap[parameter].replaceAll(os.EOL,"")
                    }
                    translatedMessage = translatedMessage.replaceAll(`%${i+1}$s`, parameter)
                }
                botSendMessage(translatedMessage.replaceAll(os.EOL,""))
            }
        }
        return
    }
    let sender = packet.source_name
    let plainMessage = packet.message
    
    console.log(message)
    if (sender != undefined) {
        botSendMessage(`${sender}: ${plainMessage}`)
    }
    
    let value = date.format(now,'YYYYMMDD');
    fs.appendFile('./'+value+'.log', message + os.EOL,function(err){
    if(err) console.log('写文件操作失败' + err);
    });
    
    let functionMessage = ""
    
    if (plainMessage == "菜单") {
        sendTip(sender, `您好,我是服务器助手，有什么可以帮您的吗？`)
        sendTip(sender, "试试输入: @给我泥土 吧！")
        sendTip(sender, "以下指令可用哦:")
        sendMsg(sender, "@设置家 x y z")
        sendMsg(sender, "@回家")
        sendMsg(sender, "@反馈问题 问题")
        sendMsg(sender, "例子：@反馈问题 钢斧(Steel Axe)没有被正确汉化")
    }
    else if (plainMessage == "@给我泥土") {
        sendMsg(sender, "既然您那么想要泥土，那我就勉为其难给您一块吧！")
        giveItem(sender, "dirt", 1)
    }
    else if (plainMessage.startsWith("@设置家")) {
        try {
            let args = plainMessage.split(' ')
            if (args.length != 4) {
                sendErr(sender, `出现了错误，请检查您的命令格式是否正确！`)
                sendMsg(sender, "@设置家 x y z")
                return
            }
            let position = {
                x: args[1],
                y: args[2],
                z: args[3]
            }
            homes[sender] = position
            var homesStr = JSON.stringify(homes);
            fs.writeFile('./homes.json', homesStr, function(err){
                if(err) console.log('写文件操作失败' + err)
            })
            sendMsg(sender, `亲爱的${sender}, 已为您成功设置家！`)
        } catch (e) {
            console.log(`出现了错误：${e}`)
            sendErr(sender, `出现了错误，请检查您的命令格式是否正确！`)
            sendMsg(sender, "@设置家 x y z")
        }
    }
    else if (plainMessage == "@回家") {
        try {
            let homePosition = homes[sender]
            if (homePosition != undefined) {
                if (Object.keys(homePosition).length == 3) {
                    tpToPosition(sender, homePosition)
                    sendMsg(sender, `亲爱的${sender}, 成功将您传送回家！`)
                }
                else {
                    sendWarn(sender, "您还没有设置家呢！请输入 @设置家 x y z 来设置家吧！")
                }
            }
            else {
                sendWarn(sender, "您还没有设置家呢！请输入 @设置家 x y z 来设置家吧！")
            }
        } catch (e) {
            console.log(`出现了错误：${e}`)
            sendErr(sender, `出现了错误，请您检查是否已经设置了家！`)
            sendMsg(sender, "@设置家 x y z")
        }
    }
    else if (plainMessage.startsWith("@反馈问题")) {
        try {
            let args = plainMessage.split(' ')
            let issue = args[1]
            if (issue != undefined && issue != "") {
                fs.appendFile('./issues.log', `[${value}](${sender})${issue}` + os.EOL, function(err){
                if(err) console.log('写文件操作失败' + err)
                })
                sendMsg(sender, "问题已成功反馈！")
            }
            else {
                sendWarn(sender, "出现了错误，请检查您的问题是否正确填写！")
            }
        } catch (e) {
            console.log(`出现了错误：${e}`)
            sendErr(sender, `出现了错误，请检查您的命令格式是否正确！`)
            sendMsg(sender, "@反馈问题 问题")
        }
        
        
    }
})

client.on('packet', (packet) => {
    if (packet.data.name == "command_output") {
        if (packet.data.params.output != undefined) {
            if (packet.data.params.output.length == 2) {
                if (packet.data.params.output[0].message_id == "commands.players.list") {
                        let players = packet.data.params.output[1].parameters;
                        botSendMessage(`当前在线的玩家:${players}`)
                }
            }
        }
    }
})